#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
import ConfigParser
from os.path import exists, relpath, abspath

sys.path.insert(0, '..')
from tests import Test, main, Provider
from libmace import files, env, distro
from libmace.providers import BaseProvider


class ConfigTest(Test):

    def test_Simple(self):
        self.touch(('1/test.d/0.conf', '[main]\nfoo=bar'))

        provider = _Provider()
        sources, __ = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, __)
        self.assertEqual(1, len(applied))
        assert provider.config.has_section('main')
        self.assertEqual('bar', provider.config.get('main', 'foo'))

    def test_GlobalVarsInDefault(self):
        self.touch(('test.d/0.conf', [
            '[DEFAULT]',
            'global = value',
            '[section1]',
            'foo=%(global)s',
            '[section2]',
            'bar=%(global)s',
            ]))

        provider = _Provider()
        sources, __ = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, __)
        self.assertEqual(1, len(applied))
        self.assertEqual('value', provider.config.get('section1', 'foo'))
        self.assertEqual('value', provider.config.get('section2', 'bar'))

    def test_GlobalVarsAreLocal(self):
        self.touch(('test.d/1.conf', [
            '[DEFAULT]',
            'global = value',
            '[section1]',
            'foo=%(global)s',
            ]))
        self.touch(('test.d/2.conf', [
            '[section2]',
            'bar=%(global)s',
            ]))

        provider = _Provider()
        sources, __ = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        self.assertRaises(ConfigParser.InterpolationMissingOptionError, files.commit, '.', 'dst', sources, __)

    def test_MultilinedOptionsInConfig(self):
        self.touch(('1/test.d/0.conf', '[main]\nfoo=b\n a\n r'))

        provider = _Provider()
        sources, __ = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, __)
        self.assertEqual(1, len(applied))
        assert provider.config.has_section('main')
        self.assertEqual('b\na\nr', provider.config.get('main', 'foo'))

    def test_ExpandVarsInConfig(self):
        os.environ['ENVAR'] = 'ENVAR_value'
        self.touch(('0.env', 'FOO=" 1 \n$ENVAR \n 2"'),
                    ('1/test.d/0.conf', '[main]\nfoo= @FOO@ '),
                   )

        provider = _Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert provider.config.has_section('main')
        self.assertEqual(' 1 \nENVAR_value \n 2', provider.config.get('main', 'foo'))

    def test_SelfExpandVarsInConfig(self):
        self.touch(('test.d/1.conf', '[main]\npre1=%(foo)s\nfoo = %(foo)s:1\npost1=%(foo)s'),
                   ('test.d/2.conf', '[main]\npre2=%(foo)s\nfoo = %(foo)s:2\npost2=%(foo)s'),
                   ('test.d/3.conf', '[main]\npre3=%(foo)s\nfoo = %(foo)s:3\npost3=%(foo)s'))

        provider = _Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert provider.config is not None
        assert provider.config.has_section('main')
        self.assertEqual(':1:2:3', provider.config.get('main', 'foo'))
        self.assertEqual(':1', provider.config.get('main', 'pre1'))
        self.assertEqual(':1', provider.config.get('main', 'post1'))
        self.assertEqual(':1:2', provider.config.get('main', 'pre2'))
        self.assertEqual(':1:2', provider.config.get('main', 'post2'))
        self.assertEqual(':1:2:3', provider.config.get('main', 'pre3'))
        self.assertEqual(':1:2:3', provider.config.get('main', 'post3'))

    def test_SeveralConfilesForTheSameFinalConfig(self):

        class _Provider(Provider):

            confiles = ['conf_9', 'conf_0']

        provider = _Provider()
        self.touch(('src/test.d/conf_0/foo.conf', 'bar'))
        self.touch(('src/test.d/conf_9/foo.conf', 'foo'))

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        files.commit('src', 'dst', sources, {})
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'foo',
                           'bar',
                           '',
                           ]),
                file('dst/etc/test.conf').read())


class _Provider(Provider):

    confpath = lambda * args: None
    config = None

    def apply(self, conftext, tran):
        self.config = tran.config
        return True


if __name__ == '__main__':
    main()
