#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
from os.path import exists, relpath, abspath

sys.path.insert(0, '..')
import tests
from tests import Test, main, Provider
from libmace import files, env, distro
from libmace.providers import BaseProvider


class FilesTest(Test):

    def test_load_Sort(self):
        self.touch(('test.d/2.conf', '2'),
                   ('test.d/1.conf', '1'),
                   ('test.d/0.conf', '0'))

        provider = Provider()
        sources, __ = files.load('.', [provider])

        assert provider in sources
        self.assertEqual(
                ['0.conf', '1.conf', '2.conf'],
                [i.filename for i in sources[provider]])

    def test_load_EarlyExpansions(self):
        self.touch(('test.d/0.conf', '@PWD@'))

        provider = Provider()
        sources, __ = files.load('.', [provider])

        assert provider in sources
        self.assertEqual(
                [[tests.tmpdir + '/test.d']],
                [i.content for i in sources[provider]])

    def test_load_Include(self):
        env.templates.value = tests.tmpdir + '/templates'

        self.touch(
                ('1/test.env', 'INCLUDE=3:' + tests.tmpdir + '/2'),
                '1/test.d/1.conf',
                '2/test.d/2.conf',
                'templates/3/test.d/3.conf',
                )

        provider = Provider()
        sources, __ = files.load('1', [provider])

        assert provider in sources
        self.assertEqual(
                ['1.conf', '2.conf', '3.conf'],
                [i.filename for i in sources[provider]])

    def test_load_IncludeGoesAtFirst(self):
        env.templates.value = tests.tmpdir + '/2'

        self.touch(
                ('1/test.env', 'INCLUDE=z'),
                ('1/a/test.d/0.conf', '1'),
                ('2/z/test.d/0.conf', '2'),
                )

        provider = Provider()
        sources, __ = files.load('1', [provider])

        assert provider in sources
        self.assertEqual(
                ['2', '1'],
                [i.content[0] for i in sources[provider]])

    def test_load_SeveralConfFiles(self):

        class _Provider(Provider):

            confiles = ['1', '2']

        self.touch(('test.d/2/0.conf', '0'),
                   ('test.d/2/1.conf', '1'),
                   ('test.d/1/2.conf', '2'),
                   ('test.d/1/3.conf', '3'))

        provider = _Provider()
        sources, __ = files.load('.', [provider])

        self.assertEqual(
                ['1/2.conf', '1/3.conf', '2/0.conf', '2/1.conf'],
                [i.filename for i in sources[provider]])

    def test_load_ConfileOrder(self):

        class _Provider(Provider):

            confiles = ['3', '2', '1']

        self.touch(('test.d/1/1.conf', '1'),
                   ('test.d/2/2.conf', '2'),
                   ('test.d/3/3.conf', '3'))

        provider = _Provider()
        sources, __ = files.load('.', [provider])

        self.assertEqual(
                ['3/3.conf', '2/2.conf', '1/1.conf'],
                [i.filename for i in sources[provider]])

    def test_load_Tree(self):
        self.touch('2/test.d/0.conf',
                   '1/test.d/3.conf',
                   '3/4/test.d/2.conf',
                   '5/6/7/test.d/1.conf',
                   )

        provider = Provider()
        sources, __ = files.load('.', [provider])

        self.assertEqual(
                ['2/test.d/0.conf', '5/6/7/test.d/1.conf', '3/4/test.d/2.conf', '1/test.d/3.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider]])

    def test_load_Override(self):
        self.touch('test.d/0.conf',
                   'test.d/1.conf',
                   '1/test.d/0.conf',
                   )

        provider = Provider()
        sources, __ = files.load('.', [provider])

        self.assertEqual(
                [('1/test.d/0.conf', True), ('test.d/0.conf', False), ('test.d/1.conf', False)],
                [(relpath(i.path, os.curdir), i.overriden) for i in sources[provider]])

    def test_load_HideFiles(self):
        self.touch(('test.d/0.conf', '#'),
                   'test.d/1.conf',
                   '1/test.d/0.conf',
                   )

        provider = Provider()
        sources, __ = files.load('.', [provider])

        self.assertEqual(
                [('1/test.d/0.conf', False, True), ('test.d/0.conf', False, False), ('test.d/1.conf', False, False)],
                [(relpath(i.path, os.curdir), i.overriden, i.hidden) for i in sources[provider]])

    def test_load_HideByServicesMask(self):
        self.touch('test-1.d/0.conf',
                   '1/test-1.d/1.conf',
                   '2/3/test-2.d/2.conf',
                   )

        provider_1 = Provider('test-1')
        provider_2 = Provider('test-2')

        env.include.value = 'test-1:test-2.d'
        sources, __ = files.load('.', [provider_1, provider_2])
        self.assertEqual(
                ['test-1.d/0.conf', '1/test-1.d/1.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_1]])
        self.assertEqual(
                ['2/3/test-2.d/2.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_2]])

        env.include.value = 'test-1'
        sources, __ = files.load('.', [provider_1, provider_2])
        self.assertEqual(
                ['test-1.d/0.conf', '1/test-1.d/1.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_1]])
        assert provider_2 not in sources

        env.include.value = 'test-2'
        sources, __ = files.load('.', [provider_1, provider_2])
        assert provider_1 not in sources
        self.assertEqual(
                ['2/3/test-2.d/2.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_2]])

        env.include.value = None
        env.exclude.value = 'test-1:test-2.d'
        sources, __ = files.load('.', [provider_1, provider_2])
        assert provider_1 not in sources
        assert provider_2 not in sources

        env.exclude.value = 'test-1'
        sources, __ = files.load('.', [provider_1, provider_2])
        assert provider_1 not in sources
        self.assertEqual(
                ['2/3/test-2.d/2.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_2]])

        env.exclude.value = 'test-2.d'
        sources, __ = files.load('.', [provider_1, provider_2])
        self.assertEqual(
                ['test-1.d/0.conf', '1/test-1.d/1.conf'],
                [relpath(i.path, os.curdir) for i in sources[provider_1]])
        assert provider_2 not in sources

    def test_load_Keywords(self):
        self.touch(('test.d/[1].conf', '1'),
                   ('test.d/[2][3].conf', '2'),
                   ('test.d/[4].conf', '3'))
        provider = Provider()

        env.keywords.value = '1'
        sources, __ = files.load('.', [provider])
        self.assertEqual(
                ['[1].conf'],
                [i.filename for i in sources[provider]])

        env.keywords.value = '2:1'
        sources, __ = files.load('.', [provider])
        self.assertEqual(
                ['[1].conf', '[2][3].conf'],
                [i.filename for i in sources[provider]])

        env.keywords.value = '3'
        sources, __ = files.load('.', [provider])
        self.assertEqual(
                ['[2][3].conf'],
                [i.filename for i in sources[provider]])

        env.keywords.value = '3:2:1:4'
        sources, __ = files.load('.', [provider])
        self.assertEqual(
                ['[1].conf', '[2][3].conf', '[4].conf'],
                [i.filename for i in sources[provider]])

    def test_load_Envars(self):

        class _Provider(Provider):

            service = 'foo'

        self.touch('foo-bar.d/0.conf',
                   ('0.env', 'FOO="1\n 2 \n3 "'),
                   )

        provider = _Provider('foo-bar')
        __, envars = files.load('.', [provider])
        self.assertEqual('1\n 2 \n3 ', envars.get('FOO'))
        self.assertEqual('/etc/init.d/foo', envars.get('FOO_BAR_SERVICE'))

    def test_commit_IsInstalled(self):

        class _Provider(Provider):

            get_requires = lambda * args: ['foo']

            def is_installed(self):
                return BaseProvider.is_installed(self)

        self.touch('test.d/0.conf')

        provider = _Provider()
        sources, __ = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        self.override(distro, 'resolve_package', lambda *args: False)
        files.commit('.', 'dst', sources, {})
        assert not exists('dst/etc/test.conf')

        self.override(distro, 'resolve_package', lambda *args: True)
        files.commit('.', 'dst', sources, {})
        assert exists('dst/etc/test.conf')

    def test_commit_ExpandVars(self):
        os.environ['ENVAR'] = 'ENVAR_value'
        self.touch(('0.env', 'FOO1=" f "\nFOO2="$ENVAR"\nFOO3="${FOO2}:${FOO1}"'),
                    ('1/test.d/0.conf', '(@FOO1@/@FOO2@/@FOO3@)'),
                   )

        provider = Provider()
        sources, envars = files.load('.', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, envars)
        self.assertEqual(1, len(applied))

        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           '( f /ENVAR_value/ENVAR_value: f )',
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_commit_SelfExpandVars(self):
        os.environ['FOO'] = 'FOO'
        self.touch(('1.env', 'V1=$FOO'),
                   ('2.env', 'V2=$FOO:$V1'),
                   ('3.env', 'V3=$FOO:$V1:$V2'))
        provider = Provider()
        __, envars = files.load('.', [provider])
        self.assertEqual('FOO', envars.get('V1'))
        self.assertEqual('FOO:FOO', envars.get('V2'))
        self.assertEqual('FOO:FOO:FOO:FOO', envars.get('V3'))

    def test_commit_DoNotExpandBeforhand(self):
        self.touch(('1.env', 'V1=$V2'),
                   ('2.env', 'V2=FAKE'),
                   )
        provider = Provider()
        __, envars = files.load('.', [provider])
        self.assertEqual('$V2', envars.get('V1'))

    def test_commit_ProviderDependiesGoesFirst(self):

        class _Provider1(Provider):

            provides = []
            get_depends = lambda * args: ['foo']
            confpath = lambda self, x: '/1.conf'

        class _Provider2(Provider):

            provides = ['foo']
            get_depends = lambda * args: ['test']
            confpath = lambda self, x: '/2.conf'

        class _Provider3(Provider):

            provides = []
            get_depends = lambda * args: ['foo']
            confpath = lambda self, x: '/3.conf'

        self.touch(
                'test.d/0.conf',
                'test1.d/1.conf',
                'test2.d/2.conf',
                'test3.d/3.conf',
                )

        provider = Provider()
        provider1 = _Provider1('test1')
        provider2 = _Provider2('test2')
        provider3 = _Provider3('test3')

        sources, __ = files.load('.', [provider1])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual(0, len(applied))
        assert not exists('dst/1.conf')

        sources, __ = files.load('.', [provider2, provider])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual(
                ['test', 'test2'],
                [i.name for i in applied])
        assert exists('dst/etc/test.conf')
        assert exists('dst/2.conf')
        os.unlink('dst/etc/test.conf')
        os.unlink('dst/2.conf')

        sources, __ = files.load('.', [provider1])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        assert not applied
        assert not exists('dst/1.conf')

        sources, __ = files.load('.', [provider2, provider1])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        assert not applied
        assert not exists('dst/1.conf')
        assert not exists('dst/2.conf')

        sources, __ = files.load('.', [provider2, provider1, provider3, provider])
        self.assertEqual(4, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual(
                ['test', 'test2', 'test1', 'test3'],
                [i.name for i in applied])
        assert exists('dst/etc/test.conf')
        assert exists('dst/1.conf')
        assert exists('dst/2.conf')

    def test_commit_ProviderDependiesTriggerDependents(self):

        class _Provider(Provider):

            get_depends = lambda * args: ['test']
            confpath = lambda self, x: '/1.conf'

        self.touch(('test.d/0.conf', 'test0'))
        self.touch(('dst/etc/test.conf', [
            '# WARNING: File was generate by mace utility, do not edit it.',
            '#          Change configuration sources and run mace.',
            '',
            'test0',
            ''
            ]))

        self.touch(('test1.d/1.conf', 'test1'))
        self.touch(('dst/1.conf', [
            '# WARNING: File was generate by mace utility, do not edit it.',
            '#          Change configuration sources and run mace.',
            '',
            'test1',
            ''
            ]))

        provider = Provider()
        provider1 = _Provider('test1')

        sources, __ = files.load('.', [provider1, provider])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        assert not applied

        os.unlink('dst/etc/test.conf')

        sources, __ = files.load('.', [provider1, provider])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual(2, len(applied))

    def test_commit_FileProviderGoesFirst(self):

        class _Provider1(Provider):

            provides = ['foo']
            confpath = lambda self, x: '/1.conf'

            def apply(self, *args):
                raise RuntimeError()

        class _Provider2(Provider):

            get_depends = lambda * args: ['foo']
            confpath = lambda self, x: '/2.conf'

        self.touch('test1.d/1.conf', 'test2.d/2.conf')

        provider1 = _Provider1('test1')
        provider2 = _Provider2('test2')

        sources, __ = files.load('.', [provider2, provider1])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual([], [i.name for i in applied])

    def test_commit_FailOnSkippedDeps(self):

        class _Provider1(Provider):

            provides = ['files']
            confpath = lambda self, x: '/1.conf'

        class _Provider2(Provider):

            provides = []
            confpath = lambda self, x: '/2.conf'

        class _Provider3(Provider):

            provides = ['files']
            confpath = lambda self, x: '/3.conf'

        self.touch('test1.d/1.conf', 'test2.d/2.conf', 'test3.d/3.conf')

        provider1 = _Provider1('test1')
        provider2 = _Provider2('test2')
        provider3 = _Provider3('test3')

        sources, __ = files.load('.', [provider2, provider1, provider3])
        self.assertEqual(3, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual(
                ['test1', 'test2', 'test3'],
                [i.name for i in applied])

    def test_commit_DefaultVars(self):
        self.touch(('src/0.env', 'V1="$SRC_ROOT"\nV2="$DST_ROOT"'),
                   ('src/test.d/1.conf', '@V1@:@V2@'),
                   )
        provider = Provider()

        env.source.value = abspath('src')
        env.output.value = abspath('dst')
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)

        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           ':'.join([env.source.value, env.output.value]),
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_commit(self):
        provider = Provider()
        self.touch(('src/test.d/foo.conf', 'foo'))

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        self.assertEqual([provider], applied)
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'foo',
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_commit_ExcludeOverriden(self):
        provider = Provider()
        self.touch(
                ('src/1/test.d/0.conf', '1'),
                ('src/2/test.d/0.conf', '2'),
                )

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        self.assertEqual([provider], applied)
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           '2',
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_commit_ExcludeHidden(self):
        provider = Provider()
        self.touch(
                ('src/1/test.d/0.conf', '1'),
                ('src/2/test.d/0.conf', ''),
                )

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        self.assertEqual([provider], applied)
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_RestartServiceOnRollback(self):
        apply_called = []

        class _Provider(Provider):

            def apply(self, *args):
                apply_called.append(True)
                raise RuntimeError()

        provider = _Provider()
        self.touch(('src/test.d/foo.conf', 'foo'))
        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        self.assertEqual(0, len(applied))
        self.assertEqual(2, len(apply_called))

    def test_commit_StartIfSourcesNotChanged(self):

        class _Provider(Provider):

            activated = []
            activate = lambda * args: _Provider.activated.append(True)
            service = 'test_daemon'

        provider = _Provider()
        self.touch(('src/test.d/foo.conf', 'foo'))

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        assert applied
        assert not _Provider.activated

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        assert not applied
        assert _Provider.activated

    def test_commit_NoSourceChanges(self):
        provider = Provider()
        self.touch(('src/test.d/1.conf', 'foo'))

        conf = '\n'.join([
            '# WARNING: File was generate by mace utility, do not edit it.',
            '#          Change configuration sources and run mace.',
            '',
            'foo',
            '',
            ])

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        self.assertEqual([provider], applied)
        assert exists('dst')
        self.assertEqual(conf, file('dst/etc/test.conf').read())

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        applied = files.commit('src', 'dst', sources, {})
        assert not applied
        assert exists('dst')
        self.assertEqual(conf, file('dst/etc/test.conf').read())

    def test_commit_Transaction(self):

        class _Provider(Provider):

            def apply(self, conftext, tran):
                raise Exception('nop')

        provider = _Provider()
        self.touch(('src/profile/test.d/foo.conf', 'bar'))

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        files.commit('src', 'dst', sources, {})
        assert not exists('dst/etc/test.conf')

        self.touch(('dst/etc/test.conf', 'foo'))
        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        files.commit('src', 'dst', sources, {})
        self.assertEqual('foo', file('dst/etc/test.conf').read())

    def test_commit_HeadersAndTailsOnlyOnceForOneFinalFile(self):

        class _Provider(Provider):

            confiles = ['1', '2', '3']
            confpath_header = lambda self, x, * args: ['header']
            footer = lambda self, x, * args: ['tail']
            confile_header = lambda self, x, * args: [x + ' header']
            confile_footer = lambda self, x, * args: [x + ' tail']
            confpath = lambda self, x: '/1.conf' if int(x) < 3 else '/2.conf'

        provider = _Provider()
        self.touch(('src/test.d/1/foo.conf', 'foo'))
        self.touch(('src/test.d/2/foo.conf', 'bar'))
        self.touch(('src/test.d/3/foo.conf', 'test'))

        sources, __ = files.load('src', [provider])
        self.assertEqual(1, len(sources))
        files.commit('src', 'dst', sources, {})
        assert exists('dst/1.conf')
        assert exists('dst/2.conf')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'header',
                           '3 header',
                           'test',
                           '3 tail',
                           'tail',
                           '',
                           ]),
                file('dst/2.conf').read())
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'header',
                           '1 header',
                           'foo',
                           '1 tail',
                           '2 header',
                           'bar',
                           '2 tail',
                           'tail',
                           '',
                           ]),
                file('dst/1.conf').read())

    def test_Transaction_ExportEnvars(self):

        class _Provider1(Provider):

            name = 'test1'
            provides = ['test1']
            confpath = lambda self, x: '/1.conf'

            def export(self, envars):
                envars['FOO'] = 'bar'

        class _Provider2(Provider):

            name = 'test2'
            provides = ['test2']
            confpath = lambda self, x: '/2.conf'

        self.touch(('src/1/test1.d/1.conf', '@FOO@'))
        self.touch(('src/2/test2.d/2.conf', '@FOO@'))

        sources, envars = files.load('src', [_Provider1('test1'), _Provider2('test2')])
        self.assertEqual(2, len(sources))
        files.commit('src', 'dst', sources, envars)
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'bar',
                           '',
                           ]),
                file('dst/1.conf').read())
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'bar',
                           '',
                           ]),
                file('dst/2.conf').read())

    def test_commit_DoConvert(self):

        class _Provider(Provider):

            def convert(self, confile, lines):
                return ['> %s' % i for i in lines]

        provider = _Provider()
        self.touch(('src/test.d/1.conf', ['1', '2', '3']))

        sources, __ = files.load('src', [provider])
        files.commit('src', 'dst', sources, {})
        assert exists('dst')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           '> 1',
                           '> 2',
                           '> 3',
                           '',
                           ]),
                file('dst/etc/test.conf').read())

    def test_commit_EmptyConfpaths(self):

        class _Provider(Provider):

            configs = {}
            applied_configs = []

            confiles = ['None', 'False']
            confpath = lambda self, x: None if x == 'None' else False

            def write_config(self, tran, confpath, lines):
                self.configs[confpath] = lines

            def apply(self, conftext, tran):
                self.applied_configs.append(tran.config)

        provider = _Provider()
        self.touch(('test.d/None/1.conf', [
            '[main]',
            'foo = bar',
            ]))
        self.touch(('test.d/False/2.conf', [
            'foo',
            'bar',
            ]))

        sources, __ = files.load('.', [provider])
        files.commit('.', 'dst', sources, {})
        self.assertEqual(
                {False: ['foo', 'bar']},
                provider.configs)
        self.assertEqual(1, len(provider.applied_configs))
        self.assertEqual(1, len(provider.applied_configs[0].sections()))
        self.assertEqual('bar', provider.applied_configs[0].get('main', 'foo'))

    def test_apply_unattended(self):
        tran = files.Transaction(tests.tmpdir, {})

        env.distributor_id.value = 'Fedora'
        files.apply_unattended(tran)
        assert exists('fsckoptions')
        self.assertEqual('-y', file('fsckoptions').read())

        self.touch(('fsckoptions', 'foo'))
        files.apply_unattended(tran)
        assert exists('fsckoptions')
        self.assertEqual('-y', file('fsckoptions').read())

        env.distributor_id.value = 'Ubuntu'
        files.apply_unattended(tran)
        assert exists('etc/default/rcS')
        self.assertEqual('FSCKFIX=yes', file('etc/default/rcS').read())

        self.touch(('etc/default/rcS', '1\n2\n'))
        files.apply_unattended(tran)
        assert exists('etc/default/rcS')
        self.assertEqual('1\n2\nFSCKFIX=yes', file('etc/default/rcS').read())


if __name__ == '__main__':
    main()
