#!/usr/bin/env python

# sugar-lint: disable

import sys
from os.path import exists

sys.path.insert(0, '..')
from tests import Test, main
from tests.providers import apache
from libmace.files import Transaction
from libmace import files, lsb_release, distro


class ProviderApacheTest(Test):

    def setUp(self):
        Test.setUp(self)
        files.uid_by_name = hash
        files.gid_by_name = hash

    def test_ensure_EnableModulesOnFedora(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')
        self.override(distro, 'resolve_package', lambda *args: True)
        self.touch(('apache.d/modules/1.conf', ['mod1 filename1', 'mod2 filename2']))
        provider = apache.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert exists('dst/etc/httpd/conf.d/zzz-mace.conf')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'LoadModule mod1 modules/filename1',
                           'LoadModule mod2 modules/filename2',
                           '',
                           ]),
                file('dst/etc/httpd/conf.d/zzz-mace.conf').read())

    def test_ensure_EnableModulesOnUbuntu(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Ubuntu')
        self.override(distro, 'resolve_package', lambda *args: True)
        self.touch(('apache.d/modules/1.conf', ['mod1 filename1', 'mod2 filename2']))
        provider = apache.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert exists('dst//etc/apache2/sites-enabled/zzz-mace.conf')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'LoadModule mod1 /usr/lib/apache2/modules/filename1',
                           'LoadModule mod2 /usr/lib/apache2/modules/filename2',
                           '',
                           ]),
                file('dst/etc/apache2/sites-enabled/zzz-mace.conf').read())

    def test_ensure_CommentsInModules(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')
        self.override(distro, 'resolve_package', lambda *args: True)
        self.touch(('apache.d/modules/1.conf', [
            '',
            '# foo',
            ' # bar',
            'mod1 filename1 # foo',
            '   mod2 filename2  ',
            '   # mod3 filename3',
            ]))
        provider = apache.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert exists('dst/etc/httpd/conf.d/zzz-mace.conf')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'LoadModule mod1 modules/filename1',
                           'LoadModule mod2 modules/filename2',
                           '',
                           ]),
                file('dst/etc/httpd/conf.d/zzz-mace.conf').read())

    def test_ensure_CommentsAndConfiguration(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')
        self.override(distro, 'resolve_package', lambda *args: True)
        self.touch(('apache.d/modules/1.conf', 'mod filename'),
                   ('apache.d/conf/2.conf', 'conf'))
        provider = apache.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert exists('dst/etc/httpd/conf.d/zzz-mace.conf')
        self.assertEqual(
                '\n'.join(['# WARNING: File was generate by mace utility, do not edit it.',
                           '#          Change configuration sources and run mace.',
                           '',
                           'LoadModule mod modules/filename',
                           'conf',
                           '',
                           ]),
                file('dst/etc/httpd/conf.d/zzz-mace.conf').read())

    def test_export(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Ubuntu')
        provider = apache.Provider()
        envars = {}
        provider.export(envars)
        assert 'HTTP_USER' in envars
        assert 'HTTP_GROUP' in envars

    def test_ensure_LogPaths(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')
        provider = apache.Provider()

        self.touch(('etc/httpd/conf.d/zzz-mace.conf', [
            'ErrorLog /foo/bar.log',
            '   CustomLog /bar/foo.log',
            ]))
        tran = Transaction('.')
        provider.ensure(None, tran)
        assert not tran.empty
        assert ('./foo/bar.log', hash('apache'), hash('apache')) in self.chown
        assert ('./bar/foo.log', hash('apache'), hash('apache')) in self.chown


if __name__ == '__main__':
    main()
