#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
import pwd
from os.path import exists, isdir

sys.path.insert(0, '..')
import tests
from tests import Test, main, Provider
from tests.providers import files as files_provider
from libmace.files import Transaction
from libmace.providers import BaseProvider
from libmace import files, util, lsb_release


class ProviderFilesTest(Test):

    def setUp(self):
        Test.setUp(self)
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')

    def set_creds_by_hash(self):
        files.uid_by_name = hash
        files.gid_by_name = hash

    def test_apply_File(self):
        self.set_creds_by_hash()
        provider = files_provider.Provider()

        config = files.ArtificialConfig()
        config.add_section(tests.tmpdir + '/1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'content', 'foo')
        provider.apply(None, tran)
        assert exists('1')
        assert (tests.tmpdir + '/1', hash('root'), hash('root')) in self.chown
        self.assertEqual('foo', file('1').read())
        del self.chown[:]
        os.unlink('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'mode', '0600')
        provider.apply(None, tran)
        assert exists('1')
        assert (tests.tmpdir + '/1', hash('root'), hash('root')) in self.chown
        self.assertEqual('foo', file('1').read())
        self.assertEqual(0600, os.stat('1').st_mode & 0700)
        del self.chown[:]
        os.unlink('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'owner', 'user')
        provider.apply(None, tran)
        assert exists('1')
        assert (tests.tmpdir + '/1', hash('user'), hash('user')) in self.chown
        self.assertEqual('foo', file('1').read())
        self.assertEqual(0600, os.stat('1').st_mode & 0700)
        del self.chown[:]
        os.unlink('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'group', 'group')
        provider.apply(None, tran)
        assert exists('1')
        assert (tests.tmpdir + '/1', hash('user'), hash('group')) in self.chown
        self.assertEqual('foo', file('1').read())
        self.assertEqual(0600, os.stat('1').st_mode & 0700)
        del self.chown[:]
        os.unlink('1')

    def test_apply_Directory(self):
        self.set_creds_by_hash()
        provider = files_provider.Provider()

        config = files.ArtificialConfig()
        config.add_section(tests.tmpdir + '/1')

        tran = Transaction('.')
        tran.config = config
        provider.apply(None, tran)
        assert isdir('1')
        assert (tests.tmpdir + '/1', hash('root'), hash('root')) in self.chown
        del self.chown[:]
        util.rmtree('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'mode', '0700')
        provider.apply(None, tran)
        assert isdir('1')
        assert (tests.tmpdir + '/1', hash('root'), hash('root')) in self.chown
        self.assertEqual(0700, os.stat('1').st_mode & 0700)
        del self.chown[:]
        util.rmtree('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'owner', 'user')
        provider.apply(None, tran)
        assert isdir('1')
        assert (tests.tmpdir + '/1', hash('user'), hash('user')) in self.chown
        self.assertEqual(0700, os.stat('1').st_mode & 0700)
        del self.chown[:]
        util.rmtree('1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'group', 'group')
        provider.apply(None, tran)
        assert isdir('1')
        assert (tests.tmpdir + '/1', hash('user'), hash('group')) in self.chown
        self.assertEqual(0700, os.stat('1').st_mode & 0700)
        del self.chown[:]
        util.rmtree('1')

    def test_apply_Exec(self):
        called = []
        self.override(util, 'assert_call', lambda cmd, **kargs: called.append(cmd))

        provider = files_provider.Provider()

        config = files.ArtificialConfig()
        config.add_section(tests.tmpdir + '/1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'content', 'foo')
        tran.config.set(tests.tmpdir + '/1', 'exec', 'bar')
        provider.apply(None, tran)
        assert exists('1')
        self.assertEqual(['bar'], called)

    def test_apply_NoReExecOnFails(self):
        called = []

        def cb(cmd, **kwargs):
            called.append(cmd)
            raise RuntimeError()

        self.override(util, 'assert_call', cb)

        provider = files_provider.Provider()

        config = files.ArtificialConfig()
        config.add_section(tests.tmpdir + '/1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'content', 'foo')
        tran.config.set(tests.tmpdir + '/1', 'exec', 'bar')
        provider.apply(None, tran)
        assert not exists('1')
        self.assertEqual(['bar'], called)

    def test_apply_ReExecOnFails(self):
        called = []

        def cb(cmd, **kwargs):
            called.append(cmd)
            raise RuntimeError()

        self.override(util, 'assert_call', cb)

        provider = files_provider.Provider()

        config = files.ArtificialConfig()
        config.add_section(tests.tmpdir + '/1')

        tran = Transaction('.')
        tran.config = config
        tran.config.set(tests.tmpdir + '/1', 'content', 'foo')
        tran.config.set(tests.tmpdir + '/1', 'exec', 'bar')
        self.touch(('1', '1'))
        provider.apply(None, tran)
        assert exists('1')
        self.assertEqual('1', file('1').read())
        self.assertEqual(['bar', 'bar'], called)

    def test_commit_FilesDependies(self):

        class _Provider(Provider):

            get_depends = lambda * args: [tests.tmpdir + '/foo']
            confpath = lambda self, x: '/test.conf'

        self.touch(('files.d/1.conf', [
            '[%s/foo]' % tests.tmpdir,
            'content=test',
            'owner=%s' % pwd.getpwuid(os.getuid()).pw_name,
            ]))
        self.touch(('test.d/2.conf', 'test'))

        provider1 = files_provider.Provider()
        provider2 = _Provider('test')

        sources, __ = files.load('.', [provider2])
        self.assertEqual(1, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual([], BaseProvider.processed_files)
        self.assertEqual([], BaseProvider.applied_files)
        assert not exists('dst/test.conf')
        self.assertEqual(0, len(applied))

        sources, __ = files.load('.', [provider1, provider2])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual([tests.tmpdir + '/foo'], BaseProvider.processed_files)
        self.assertEqual([tests.tmpdir + '/foo'], BaseProvider.applied_files)
        assert exists('dst/test.conf')
        self.assertEqual(2, len(applied))

    def test_commit_FileDependiesTriggerDependents(self):

        class _Provider(Provider):

            provides = []
            get_depends = lambda * args: [tests.tmpdir + '/foo']
            confpath = lambda self, x: '/test.conf'

        self.touch(('files.d/1.conf', [
            '[%s/foo]' % tests.tmpdir,
            'content=test',
            'owner=%s' % pwd.getpwuid(os.getuid()).pw_name,
            ]))
        self.touch(('test.d/2.conf', 'test'))

        self.touch((tests.tmpdir + '/foo', 'test'))
        self.touch(('dst/test.conf', [
            '# WARNING: File was generate by mace utility, do not edit it.',
            '#          Change configuration sources and run mace.',
            '',
            'test',
            '',
            ]))

        provider1 = files_provider.Provider()
        provider2 = _Provider('test')

        sources, __ = files.load('.', [provider1, provider2])
        self.assertEqual(2, len(sources))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual([tests.tmpdir + '/foo'], BaseProvider.processed_files)
        self.assertEqual([], BaseProvider.applied_files)
        self.assertEqual(0, len(applied))
        BaseProvider.processed_files = []

        self.touch((tests.tmpdir + '/foo', 'fake'))
        applied = files.commit('.', 'dst', sources, {})
        self.assertEqual([tests.tmpdir + '/foo'], BaseProvider.processed_files)
        self.assertEqual([tests.tmpdir + '/foo'], BaseProvider.applied_files)
        self.assertEqual(2, len(applied))


if __name__ == '__main__':
    main()
